@@ -35,26 +35,26 @@ opts = {
     "perform_checks": False,
     "pfix": None,
     "axis": 1,
-    "min_dh_bound": 10.0,
-    "max_dh_bound": 170.0,
+    "min_dh_angle_bound": 10.0,
+    "max_dh_angle_bound": 170.0,
     "points": None,
+    "delta_t": 0.20,
 }
 
 
 def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
-    r"""Improve an existing 3D mesh by removing degenerate elements called
+    r"""Improve an existing 3D mesh by removing degenerate elements
     commonly referred to as `slivers`.
 
     :param points:
-        The name of a SEG-y or binary file containing a seismic velocity model
-    :type filename: ``string``
+        An array of points that describe the vertices of an existing (higher-quality) mesh.
+    :type points: `np.ndarray`
     :param domain:
-        Either a :class:`geometry` or a function that takes a point and returns the
-        signed nearest distance to the domain boundary Ω
-    :type domain: A :class:`geometry` object (e.g., Rectangle, Cube, or Circle) or a function object.
+        A function that takes a point and returns the signed nearest distance to the domain boundary Ω
+    :type domain: A :class:`geometry.Rectangle/Cube/Circle` object or a function object.
     :param cell_size:
-        Either a :class:`size_function` object or a function that can evalulate a point and return a mesh size.
-    :type cell_size: A :class:`cell_size` object or a function object.
+        A function that can evalulate a point and return a mesh size.
+    :type cell_size: A :class:`SizeFunction` object or a function object.
     :param h0:
         The minimum element size in the domain
     :type h0: `float`
@@ -70,23 +70,22 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
             Output to the screen `nscreen` timestep. (default==1)
         * *max_iter* (``float``) --
             Maximum number of meshing iterations. (default==50)
-        * *seed* (``float`` or ``int``) --
-            Psuedo-random seed to initialize meshing points.
         * *perform_checks* (`boolean`) --
             Whether or not to perform mesh linting/mesh cleanup. (default==False)
         * *pfix* (`array-like`) --
             An array of points to constrain in the mesh. (default==None)
         * *axis* (`int`) --
             The axis to decompose the mesh (1,2, or 3). (default==1)
-        * *min_dh_bound* (`float`) --
+        * *min_dh_angle_bound* (`float`) --
             The minimum allowable dihedral angle bound. (default==10 degrees)
-        * *max_dh_bound* (`float`) --
+        * *max_dh_angle_bound* (`float`) --
             The maximum allowable dihedral angle bound. (default==170 degrees)
 
     """
     comm = comm or MPI.COMM_WORLD
     if comm.rank > 0:
-        warnings.warn("Sliver removal only works in serial for now")
+        if comm.rank == 1:
+            warnings.warn("Sliver removal only works in serial for now")
         return True, True
 
     opts.update(kwargs)
@@ -94,7 +93,8 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
 
     dim = points.shape[1]
     if dim == 2:
-        raise Exception("Mesh improvement currently on works in 3D")
+        if comm.rank == 0:
+            raise Exception("Mesh improvement currently on works in 3D")
 
     # unpack domain
     fd, bbox0 = _unpack_domain(domain)
@@ -104,17 +104,35 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
     # take maxmin of boxes
     bbox = _minmax(bbox0, bbox1)
 
+    # rebuild the Rectangle or cube if domain padding
+    if bbox0 != bbox1:
+        fd = geometry.Cube(bbox).eval
+
     if not isinstance(bbox, tuple):
         raise ValueError("`bbox` must be a tuple")
 
     if opts["max_iter"] < 0:
         raise ValueError("`max_iter` must be > 0")
     max_iter = opts["max_iter"]
+    print(
+        "Will attempt " + str(max_iter) + " iterations to bound the dihedral angles..."
+    )
 
     geps = 1e-1 * h0
     deps = np.sqrt(np.finfo(np.double).eps) * h0
-    min_dh_bound = opts["min_dh_bound"] * math.pi / 180
-    max_dh_bound = opts["max_dh_bound"] * math.pi / 180
+    min_dh_bound = opts["min_dh_angle_bound"] * math.pi / 180
+    max_dh_bound = opts["max_dh_angle_bound"] * math.pi / 180
+
+    print(
+        "Enforcing a min. dihedral bound of: "
+        + str(min_dh_bound * 180 / math.pi)
+        + " degrees..."
+    )
+    print(
+        "Enforcing a max. dihedral bound of: "
+        + str(max_dh_bound * 180 / math.pi)
+        + " degrees..."
+    )
 
     DT = _select_cgal_dim(dim)
 
@@ -175,7 +193,12 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
             move = t[ele_nums, 0]
             num_move = move.size
             if num_move == 0:
-                print("Termination reached...no slivers detected!", flush=True)
+                print(
+                    "Termination reached in "
+                    + str(count)
+                    + " iterations...no slivers detected!",
+                    flush=True,
+                )
                 p, t, _ = geometry.fix_mesh(p, t, dim=dim, delete_unused=True)
                 return p, t
 
@@ -202,7 +225,7 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
 
         # Number of iterations reached, stop.
         if count == (max_iter - 1):
-            p, t = _termination(p, t, opts, comm)
+            p, t = _termination(p, t, opts, comm, sliver=True)
             break
 
         count += 1
@@ -214,19 +237,18 @@ def sliver_removal(points, domain, cell_size, h0, comm=None, **kwargs):
     return p, t
 
 
+# @profile
 def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
-    r"""Generate a 2D/3D triangulation using callbacks to a sizing function `cell_size`
-       and signed distance function :class:`domain`
+    r"""Generate a 2D/3D triangulation using callbacks to a sizing function `cell_size` and signed distance function :class:`domain`
 
     :param domain:
-        Either a :class:`geometry` or a function that takes a point and returns the
-        signed nearest distance to the domain boundary Ω
-    :type domain: A :class:`geometry` object (e.g., Rectangle, Cube, or Circle) or a function object.
+        A function that takes a point and returns the signed nearest distance to the domain boundary Ω.
+    :type domain: A :class:`geometry.Rectangle/Cube/Circle` object or a function object.
     :param cell_size:
         Either a :class:`size_function` object or a function that can evalulate a point and return a mesh size.
     :type cell_size: A :class:`cell_size` object or a function object.
     :param h0:
-        The minimum element size in the domain
+        The minimum element size in the domain.
     :type h0: `float`
     :param comm:
         MPI4py communicator
@@ -237,19 +259,21 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
 
     :Keyword Arguments:
         * *bbox* (``tuple``) --
-            Bounding box containing domain extents.
+            Bounding box containing domain extents. REQUIRED IF NOT USING :class:`cell_size`
         * *nscreen* (``float``) --
             Output to the screen `nscreen` timestep. (default==1)
         * *max_iter* (``float``) --
             Maximum number of meshing iterations. (default==50)
         * *seed* (``float`` or ``int``) --
-            Psuedo-random seed to initialize meshing points.
+            Psuedo-random seed to initialize meshing points. (default==0)
         * *perform_checks* (`boolean`) --
             Whether or not to perform mesh linting/mesh cleanup. (default==False)
         * *pfix* (`array-like`) --
             An array of points to constrain in the mesh. (default==None)
         * *axis* (`int`) --
             The axis to decompose the mesh (1,2, or 3). (default==1)
+        * *delta_t* (`float`) --
+            Psuedo-timestep to control movement of points (default=0.10)
 
     :return: points: vertex coordinates of mesh
     :rtype: points: (numpy.ndarray[`float` x dim])
@@ -276,6 +300,16 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
 
     # check bbox shape
     dim = int(len(bbox) / 2)
+
+    # rebuild the Rectangle or Cube if domain padding
+    if bbox0 != bbox1:
+        if dim == 2:
+            tmp = geometry.Rectangle(bbox)
+        elif dim == 3:
+            tmp = geometry.Cube(bbox)
+
+        fd = tmp.eval
+
     bbox = np.array(bbox).reshape(-1, 2)
 
     # check h0
@@ -286,11 +320,9 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         raise ValueError("`max_iter` must be > 0")
     max_iter = opts["max_iter"]
 
-    np.random.seed(opts["seed"])
-
     """These parameters originate from the original DistMesh"""
     L0mult = 1 + 0.4 / 2 ** (dim - 1)
-    delta_t = 0.1
+    delta_t = opts["delta_t"]
     geps = 1e-1 * h0
     deps = np.sqrt(np.finfo(np.double).eps) * h0
 
@@ -329,30 +361,25 @@ def generate_mesh(domain, cell_size, h0, comm=None, **kwargs):
         # Remove points outside the domain
         t = _remove_triangles_outside(p, t, fd, geps)
 
-        # Compute the forces on the bars
-        Ftot = _compute_forces(p, t, fh, h0, L0mult)
-
-        Ftot[:nfix] = 0  # Force = 0 at fixed points
-
-        # Last timestep in parallel, we don't move points
-        if comm.size > 1:
-            if count < max_iter - 1:
-                p += delta_t * Ftot
-        else:
-            p += delta_t * Ftot  # Update positions
-
-        # Bring outside points back to the boundary
-        p = _project_points_back(p, fd, deps)
-
         # Number of iterations reached, stop.
         if count == (max_iter - 1):
             p, t = _termination(p, t, opts, comm)
             break
 
+        # Compute the forces on the edges
+        Ftot = _compute_forces(p, t, fh, h0, L0mult)
+
+        Ftot[:nfix] = 0  # Force = 0 at fixed points
+
+        # Update positions
+        p += delta_t * Ftot
+
+        # Bring outside points back to the boundary
+        p = _project_points_back(p, fd, deps)
+
         if comm.size > 1:
             # If continuing on, delete ghost points
             p = np.delete(p, inv[-recv_ix::], axis=0)
-            comm.barrier()
 
         # Show the user some progress so they know something is happening
         if count % nscreen == 0 and comm.rank == 0:
@@ -401,6 +428,9 @@ def _unpack_domain(domain):
     elif isinstance(domain, geometry.Cube):
         bbox = (domain.x1, domain.x2, domain.y1, domain.y2, domain.z1, domain.z2)
         fd = domain.eval
+    elif isinstance(domain, geometry.Circle):
+        bbox = (domain.x1, domain.x2, domain.y1, domain.y2)
+        fd = domain.eval
     elif callable(domain):
         # get the bbox from the name value pairs or quit
         bbox = opts["bbox"]
@@ -423,6 +453,9 @@ def _parse_kwargs(kwargs):
             "domain",
             "cell_size",
             "bbox",
+            "min_dh_angle_bound",
+            "max_dh_angle_bound",
+            "delta_t",
         }:
             pass
         else:
@@ -440,50 +473,51 @@ def _display_progress(p, t, count, nscreen, maxdp, comm):
     )
 
 
-def _termination(p, t, opts, comm):
+def _termination(p, t, opts, comm, sliver=False):
     """Shut it down when reacing `max_iter`"""
     dim = p.shape[1]
     if comm.rank == 0:
         print("Termination reached...maximum number of iterations reached.", flush=True)
-    if comm.size > 1:
+    if comm.size > 1 and sliver is False:
         # gather onto rank 0
         p, t = migration.aggregate(p, t, comm, comm.size, comm.rank, dim=dim)
     # perform linting if asked
     if comm.rank == 0 and opts["perform_checks"]:
         p, t = geometry.linter(p, t, dim=dim)
+    elif comm.rank == 0:
+        p, t, _ = geometry.fix_mesh(p, t, dim=dim, delete_unused=True)
     return p, t
 
 
-def _get_bars(t):
+def _get_edges(t):
     """Describe each bar by a unique pair of nodes"""
     dim = t.shape[1] - 1
-    bars = np.concatenate([t[:, [0, 1]], t[:, [1, 2]], t[:, [2, 0]]])
+    edges = np.concatenate([t[:, [0, 1]], t[:, [1, 2]], t[:, [2, 0]]])
     if dim == 3:
-        bars = np.concatenate((bars, t[:, [0, 3]], t[:, [1, 3]], t[:, [2, 3]]), axis=0)
-    bars = mutils.unique_rows(
-        np.ascontiguousarray(bars, dtype=np.uint32)
-    )  # Bars as node pairs
-    return bars
+        edges = np.concatenate(
+            (edges, t[:, [0, 3]], t[:, [1, 3]], t[:, [2, 3]]), axis=0
+        )
+    return geometry.unique_edges(edges)
 
 
+# @profile
 def _compute_forces(p, t, fh, h0, L0mult):
     """Compute the forces on each edge based on the sizing function"""
     dim = p.shape[1]
     N = p.shape[0]
-    bars = _get_bars(t)
-    barvec = p[bars[:, 0]] - p[bars[:, 1]]  # List of bar vectors
+    edges = _get_edges(t)
+    barvec = p[edges[:, 0]] - p[edges[:, 1]]  # List of bar vectors
     L = np.sqrt((barvec ** 2).sum(1))  # L = Bar lengths
     L[L == 0] = np.finfo(float).eps
-    hbars = fh(p[bars].sum(1) / 2)
-    L0 = hbars * L0mult * ((L ** dim).sum() / (hbars ** dim).sum()) ** (1.0 / dim)
+    hedges = fh(p[edges].sum(1) / 2)
+    L0 = hedges * L0mult * ((L ** dim).sum() / (hedges ** dim).sum()) ** (1.0 / dim)
     F = L0 - L
     F[F < 0] = 0  # Bar forces (scalars)
     Fvec = (
         F[:, None] / L[:, None].dot(np.ones((1, dim))) * barvec
     )  # Bar forces (x,y components)
-
     Ftot = mutils.dense(
-        bars[:, [0] * dim + [1] * dim],
+        edges[:, [0] * dim + [1] * dim],
         np.repeat([list(range(dim)) * 2], len(F), axis=0),
         np.hstack((Fvec, -Fvec)),
         shape=(N, dim),
@@ -567,8 +601,13 @@ def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
         p = mutils.make_init_points(bbox, comm.rank, comm.size, opts["axis"], h0, dim)
     else:
         # Create initial distribution in bounding box (equilateral triangles)
-        p = np.mgrid[tuple(slice(min, max + h0, h0) for min, max in bbox)].astype(float)
-        p = p.reshape(dim, -1).T
+        if dim == 2:
+            p = mutils.create_staggered_grid_2d(h0, bbox)
+        elif dim == 3:
+            p = np.mgrid[tuple(slice(min, max + h0, h0) for min, max in bbox)].astype(
+                float
+            )
+            p = p.reshape(dim, -1).T
 
     # Remove points outside the region, apply the rejection method
     p = p[fd(p) < geps]  # Keep only d<0 points
@@ -577,6 +616,7 @@ def _generate_initial_points(h0, geps, dim, bbox, fh, fd, pfix, comm, opts):
     # Make sure decimation occurs uniformly accross ranks
     if comm.size > 1:
         r0m = comm.allreduce(r0m, op=MPI.MIN)
+    np.random.seed(opts["seed"])
     p = np.vstack(
         (
             pfix,
