@@ -4,6 +4,7 @@ from .cpp.fast_geometry import (
     calc_circumsphere_grad,
     calc_dihedral_angles,
     calc_volume_grad,
+    unique_edges,
 )
 from .signed_distance_functions import Rectangle, Cube, Circle, drectangle, dblock
 from .utils import (
@@ -40,7 +41,6 @@ __all__ = [
     "Circle",
     "drectangle",
     "dblock",
-    "dintersect",
     "calc_dihedral_angles",
     "do_any_overlap",
     "linter",
@@ -61,4 +61,5 @@ __all__ = [
     "get_boundary_facets",
     "get_winded_boundary_edges",
     "vertex_in_entity3",
+    "unique_edges",
 ]
